@@ -13,11 +13,6 @@
 #ifndef SAMA5D3_H
 #define SAMA5D3_H
 
-/*
- * defines to be used in other places
- */
-#define CONFIG_AT91FAMILY	/* it's a member of AT91 */
-
 /*
  * Peripheral identifiers/interrupts.
  */
@@ -188,8 +183,6 @@
  * Other misc defines
  */
 #define ATMEL_PIO_PORTS		5
-#define CPU_HAS_PIO3
-#define PIO_SCDR_DIV		0x3fff
 #define CPU_HAS_PCR
 
 /* Timer */
