@@ -8,7 +8,6 @@
 
 #ifndef _ASM_CONFIG_H_
 #define _ASM_CONFIG_H_
-
-#define CONFIG_NEEDS_MANUAL_RELOC
+#define CONFIG_LMB
 
 #endif
