@@ -164,21 +164,6 @@ int ddr3_tip_init_config_func(u32 dev_num,
 	return MV_OK;
 }
 
-/*
- * Read training result table
- */
-int hws_ddr3_tip_read_training_result(
-	u32 dev_num, enum hws_result result[MAX_STAGE_LIMIT][MAX_INTERFACE_NUM])
-{
-	dev_num = dev_num;
-
-	if (result == NULL)
-		return MV_BAD_PARAM;
-	memcpy(result, training_result, sizeof(result));
-
-	return MV_OK;
-}
-
 /*
  * Get training result info pointer
  */
@@ -342,8 +327,6 @@ int ddr3_tip_print_log(u32 dev_num, u32 mem_addr)
 	u32 if_id = 0;
 	struct hws_topology_map *tm = ddr3_get_topology_map();
 
-	mem_addr = mem_addr;
-
 #ifndef EXCLUDE_SWITCH_DEBUG
 	if ((is_validate_window_per_if != 0) ||
 	    (is_validate_window_per_pup != 0)) {
@@ -835,7 +818,6 @@ static int ddr3_tip_access_atr(u32 dev_num, u32 flag_id, u32 value, u32 **ptr)
 	u32 tmp_val = 0, if_id = 0, pup_id = 0;
 	struct hws_topology_map *tm = ddr3_get_topology_map();
 
-	dev_num = dev_num;
 	*ptr = NULL;
 
 	switch (flag_id) {
@@ -1184,8 +1166,6 @@ int print_adll(u32 dev_num, u32 adll[MAX_INTERFACE_NUM * MAX_BUS_NUM])
 	u32 i, j;
 	struct hws_topology_map *tm = ddr3_get_topology_map();
 
-	dev_num = dev_num;
-
 	for (j = 0; j < tm->num_of_bus_per_interface; j++) {
 		VALIDATE_ACTIVE(tm->bus_act_mask, j);
 		for (i = 0; i < MAX_INTERFACE_NUM; i++) {
@@ -1244,8 +1224,6 @@ int ddr3_tip_sweep_test(u32 dev_num, u32 test_type,
 	u32 reg_addr = 0;
 	struct hws_topology_map *tm = ddr3_get_topology_map();
 
-	mem_addr = mem_addr;
-
 	if (test_type == 0) {
 		reg_addr = 1;
 		ui_mask_bit = 0x3f;
@@ -1316,8 +1294,6 @@ int ddr3_tip_run_sweep_test(int dev_num, u32 repeat_num, u32 direction,
 	u32 max_cs = hws_ddr3_tip_max_cs_get();
 	struct hws_topology_map *tm = ddr3_get_topology_map();
 
-	repeat_num = repeat_num;
-
 	if (mode == 1) {
 		/* per pup */
 		start_pup = 0;
