@@ -5,39 +5,12 @@
  * SPDX-License-Identifier:	GPL-2.0+
  */
 
-#include <dm/device.h>
+#include <common.h>
+#include <dm.h>
 #include <dm/pinctrl.h>
 
 #include "pinctrl-uniphier.h"
 
-static const struct uniphier_pinctrl_pin uniphier_sld8_pins[] = {
-	UNIPHIER_PINCTRL_PIN(32, 8),
-	UNIPHIER_PINCTRL_PIN(33, 8),
-	UNIPHIER_PINCTRL_PIN(34, 8),
-	UNIPHIER_PINCTRL_PIN(35, 8),
-	UNIPHIER_PINCTRL_PIN(36, 8),
-	UNIPHIER_PINCTRL_PIN(37, 8),
-	UNIPHIER_PINCTRL_PIN(38, 8),
-	UNIPHIER_PINCTRL_PIN(39, 8),
-	UNIPHIER_PINCTRL_PIN(40, 9),
-	UNIPHIER_PINCTRL_PIN(41, 0),
-	UNIPHIER_PINCTRL_PIN(42, 0),
-	UNIPHIER_PINCTRL_PIN(43, 0),
-	UNIPHIER_PINCTRL_PIN(44, 0),
-	UNIPHIER_PINCTRL_PIN(70, 0),
-	UNIPHIER_PINCTRL_PIN(71, 0),
-	UNIPHIER_PINCTRL_PIN(102, 10),
-	UNIPHIER_PINCTRL_PIN(103, 10),
-	UNIPHIER_PINCTRL_PIN(104, 11),
-	UNIPHIER_PINCTRL_PIN(105, 11),
-	UNIPHIER_PINCTRL_PIN(108, 13),
-	UNIPHIER_PINCTRL_PIN(109, 13),
-	UNIPHIER_PINCTRL_PIN(112, 0),
-	UNIPHIER_PINCTRL_PIN(113, 0),
-	UNIPHIER_PINCTRL_PIN(114, 0),
-	UNIPHIER_PINCTRL_PIN(115, 0),
-};
-
 static const unsigned emmc_pins[] = {21, 22, 23, 24, 25, 26, 27};
 static const int emmc_muxvals[] = {1, 1, 1, 1, 1, 1, 1};
 static const unsigned emmc_dat8_pins[] = {28, 29, 30, 31};
@@ -142,8 +115,6 @@ static const char * const uniphier_sld8_functions[] = {
 };
 
 static struct uniphier_pinctrl_socdata uniphier_sld8_pinctrl_socdata = {
-	.pins = uniphier_sld8_pins,
-	.pins_count = ARRAY_SIZE(uniphier_sld8_pins),
 	.groups = uniphier_sld8_groups,
 	.groups_count = ARRAY_SIZE(uniphier_sld8_groups),
 	.functions = uniphier_sld8_functions,
